@@ -46,6 +46,7 @@ stl_open(stl_file *stl, char *file) {
 void
 stl_initialize(stl_file *stl) {
   stl->error = 0;
+  stl->stats.backwards_edges = 0;
   stl->stats.degenerate_facets = 0;
   stl->stats.edges_fixed  = 0;
   stl->stats.facets_added = 0;
@@ -77,8 +78,8 @@ stl_count_facets(stl_file *stl, char *file) {
 
   if (stl->error) return;
 
-  /* Open the file */
-  stl->fp = fopen(file, "r");
+  /* Open the file in binary mode first */
+  stl->fp = fopen(file, "rb");
   if(stl->fp == NULL) {
     error_msg = (char*)
                 malloc(81 + strlen(file)); /* Allow 80 chars+file size for message */
@@ -104,9 +105,6 @@ stl_count_facets(stl_file *stl, char *file) {
   for(s = 0; s < sizeof(chtest); s++) {
     if(chtest[s] > 127) {
       stl->stats.type = binary;
-      /* close and reopen with binary flag (needed on Windows) */
-      fclose(stl->fp);
-      stl->fp = fopen(file, "rb");
       break;
     }
   }
@@ -137,6 +135,13 @@ stl_count_facets(stl_file *stl, char *file) {
   }
   /* Otherwise, if the .STL file is ASCII, then do the following */
   else {
+    /* Reopen the file in text mode (for getting correct newlines on Windows) */
+    if (freopen(file, "r", stl->fp) == NULL) {
+      perror("Could not reopen the file, something went wrong");
+      stl->error = 1;
+      return;
+    }
+
     /* Find the number of facets */
     j = 0;
     for(i = 0; i < file_size ; i++) {
@@ -175,7 +180,7 @@ stl_allocate(stl_file *stl) {
   /* Allocate memory for the neighbors list */
   stl->neighbors_start = (stl_neighbors*)
                          calloc(stl->stats.number_of_facets, sizeof(stl_neighbors));
-  if(stl->facet_start == NULL) perror("stl_initialize");
+  if(stl->neighbors_start == NULL) perror("stl_initialize");
 }
 
 void
